@@ -29,26 +29,16 @@ from nose.tools import raises
 from nose import SkipTest
 from cmd import Command
 
-restore_cmd='trash-restore'
-legacy_restore_cmd='restore-trash'
-
 # Peraphs TODO: Refactoring: move cmd(), create_file(), trash(), trash-dir to
 # a Sandbox class.
 class RestoreTest(TestCase):
-    def cmd(self, *cmdline) :
-        """
-        Create a command with the current enviroment (self.env)
-        """
-        return Command(cmdline, self.env)
-
     def setUp(self):
-        self.sandbox = Path("./sandbox")
+        from common import create_cmdenv
+        self.cmdenv = create_cmdenv()
+
+        self.sandbox = Path("./sandbox").absolute()
         self.sandbox.remove()
         self.sandbox.mkdirs()
-        self.env = {
-            'HOME':'./sandbox/home',
-            'PATH':Path(__file__).parent.parent.parent.join('scripts')
-        }
         self.trashdir = HomeTrashDirectory(
             Path('./sandbox/home/.local/share/Trash'))
 
@@ -73,19 +63,14 @@ class RestoreTest(TestCase):
         assert not path.exists()
         return result
 
-    def test_cmd_creates_commands(self):
-        result = self.cmd("/bin/echo", "pippo").run()
-        assert_equals("",result.err_data)
-        assert_equals("pippo\n", result.out_data)
-        assert_equals(0,result.exit_code)
-
     def test_version_option(self):
         """
         $ trash-restore --version
         0.2.1
         """
+        raise SkipTest("trash-restore not yet ready")
         import re
-        result = self.cmd(restore_cmd,'--version').assert_succeed()
+        result = self.cmdenv.run('trash-restore','--version').assert_succeed()
         assert_equals("", result.err_data)
         expected = re.compile("trash-restore (\d)+\.(\d)+\.(\d)+")
         assert expected.match(result.out_data) is not None
@@ -100,6 +85,7 @@ class RestoreTest(TestCase):
         $ trash-list
         1977-01-12 12:00:00 /home/andrea/file # the oldest remain in trashcan
         """
+        raise SkipTest("trash-restore not yet ready")
 
         # prepare
         foo_file = self.create_file('foo', "first")
@@ -119,7 +105,7 @@ class RestoreTest(TestCase):
         print trashed_file3.deletion_date
 
         # execute
-        self.cmd(restore_cmd,foo_file.absolute()).assert_succeed()
+        self.cmd("trash-restore",foo_file.absolute()).assert_succeed()
         assert_true(foo_file.exists()) # File has been restored ?
         assert_equals("latest", foo_file.read()) # Is the latest deleted file?
 
@@ -130,13 +116,15 @@ class RestoreTest(TestCase):
         $ cd /home/andrea
         $ trash-restore ./file
         """
+        raise SkipTest("trash-restore not yet ready")
+
         # prepare
         foo_file = self.create_file('file', "content")
         self.trash(foo_file)
         assert_false(foo_file.exists())
 
         # execute
-        self.cmd(restore_cmd,"./sandbox/file").assert_succeed()
+        self.cmdenv.run("trash-restore","./sandbox/file").assert_succeed()
         assert_true(foo_file.exists()) # File has been restored ?
         assert_equals("content", foo_file.read()) # Is the latest deleted file?
 
@@ -148,7 +136,7 @@ class RestoreTest(TestCase):
         """
         raise SkipTest()
         # execute
-        result = self.cmd(restore_cmd, 'non-existent').assert_fail()
+        result = self.cmd(self.restore_cmd, 'non-existent').assert_fail()
         # test
         assert_equals(result.exit_code, 1)
         assert_equals(result.out_data, "")
@@ -169,7 +157,7 @@ class RestoreTest(TestCase):
         self.create_file('existing-file')
 
         # execute
-        result = self.cmd(restore_cmd, 'existing-file').assert_fail()
+        result = self.cmdenv.run('trash-restore', 'existing-file').assert_fail()
 
         # test
         assert_equals(result.exit_code, 1)
@@ -190,8 +178,8 @@ class RestoreTest(TestCase):
         self.create_file('existing-file')
 
         # execute
-        result = self.cmd(restore_cmd, '--force',
-                          'existing-file').assert_succeed
+        result = self.cmdenv.run('trash-restore', '--force',
+                                 'existing-file').assert_succeed
 
         # test
         assert_equals(result.exit_code, 0)
@@ -206,7 +194,7 @@ class RestoreTest(TestCase):
         ...
         """
         raise SkipTest()
-        result = self.cmd(restore_cmd, '--help').assert_succeed()
+        result = self.cmdenv.run('trash-restore', '--help').assert_succeed()
         assert_equals(result.exit_code, 0)
         assert_equals(result.out_data,
 """Usage: trash-restore TRASHED-FILE [NEW-LOCATION]
@@ -231,8 +219,23 @@ Options:
         self.trash(Path('sandbox/dir/file'))
         self.sandbox.join('dir').remove()
 
-        result = self.cmd(legacy_restore_cmd).assert_succeed("0")
-        assert result.err_data == ""
-        print result.out_data
-        assert result.exit_code == 0
-        assert self.sandbox.join("dir").join("file").exists()
+        result=self.cmdenv.cmd('restore-trash').run("0")
+        result.assert_result(
+            exit_code=0,
+            error=""
+        )
+
+        assert_true(self.sandbox.child("dir").child("file").exists())
+
+    def test_overwrite(self):
+        # Trash a file, then create a file of the same name in its place.
+        # Restore the file, and ensure that it asks before overwriting.
+        self.sandbox.join('testfile').touch()
+        self.trash(Path('sandbox/testfile'))
+        self.sandbox.join('testfile').touch()
+        result = self.cmdenv.cmd('restore-trash').run('0')
+        result.assert_result(
+            exit_code=1,
+            error='Refusing to overwrite existing file "testfile".\n',
+        )
+
