@@ -31,18 +31,25 @@ from nose import SkipTest
 
 class TestVolume(unittest.TestCase) :
     def test_all(self) :
-        if sys.platform[:3] != "win":
-            volumes = Volume.all()
-            self.assert_(len(list(volumes)) > 0)
-            for v in volumes:
-                self.assert_(isinstance(v, Volume))
+
+	volumes = Volume.all()
+	self.assert_(len(list(volumes)) > 0)
+	for v in volumes:
+	    self.assert_(isinstance(v, Volume))
 
     def testCmpVolumes(self) :
         v1 = Volume(Path(os.sep))
         v2 = Volume(Path(os.sep))
 
         self.assert_(v1 == v2)
-        
+
+def _is_mac_os():
+    return sys.platform == 'darwin'
+
+def _is_cygwin():
+    return sys.platform == 'cygwin' 
+
+
 class TestPath(unittest.TestCase) :
     def test_constructor(self) :
         instance = Path("dummy/path")
